@@ -1,8 +1,8 @@
 /* 
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2008 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2007 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -24,11 +24,16 @@
  */
 
 #include "orte_config.h"
+#include "orte/constants.h"
+
 #include "opal/opal_socket_errno.h"
 
-#include "orte/class/orte_proc_table.h"
-#include "orte/orte_constants.h"
-#include "orte/mca/ns/ns.h"
+#include "opal/class/opal_hash_table.h"
+#include "orte/util/name_fns.h"
+#include "orte/mca/errmgr/errmgr.h"
+#include "orte/mca/routed/routed.h"
+#include "orte/runtime/orte_globals.h"
+
 #include "orte/mca/oob/tcp/oob_tcp.h"
 #include "orte/mca/oob/tcp/oob_tcp_msg.h"
 
@@ -40,7 +45,6 @@ static bool mca_oob_tcp_msg_recv(mca_oob_tcp_msg_t* msg, mca_oob_tcp_peer_t* pee
 static void mca_oob_tcp_msg_data(mca_oob_tcp_msg_t* msg, mca_oob_tcp_peer_t* peer);
 static void mca_oob_tcp_msg_ping(mca_oob_tcp_msg_t* msg, mca_oob_tcp_peer_t* peer);
 
-
 OBJ_CLASS_INSTANCE(
     mca_oob_tcp_msg_t,
     opal_list_item_t,
@@ -107,6 +111,7 @@ int mca_oob_tcp_msg_wait(mca_oob_tcp_msg_t* msg, int* rc)
            time. */
         opal_progress();
         opal_event_loop(OPAL_EVLOOP_NONBLOCK);
+        OPAL_CR_TEST_CHECKPOINT_READY();
     }
 #endif
 
@@ -180,41 +185,65 @@ int mca_oob_tcp_msg_complete(mca_oob_tcp_msg_t* msg, orte_process_name_t * peer)
     OPAL_THREAD_LOCK(&msg->msg_lock);
     msg->msg_complete = true;
     if(NULL != msg->msg_cbfunc) {
-        opal_list_item_t* item;
         OPAL_THREAD_UNLOCK(&msg->msg_lock);
 
-        /* post to a global list of completed messages */
+#if defined(__WINDOWS__)
+        /**
+         * In order to be able to generate TCP events recursively, Windows need
+         * to get out of the callback attached to a specific socket. Therefore,
+         * as our OOB allow to block on a connection from a callback on the same
+         * connection, we have to trigger the completion callbacks outside of the
+         * OOB callbacks. We add them to the completed list here, and the progress
+         * engine will call our progress function later once all socket related
+         * events have been processed.
+         */
         OPAL_THREAD_LOCK(&mca_oob_tcp_component.tcp_lock);
         opal_list_append(&mca_oob_tcp_component.tcp_msg_completed, (opal_list_item_t*)msg);
-        if(opal_list_get_size(&mca_oob_tcp_component.tcp_msg_completed) > 1) {
-            OPAL_THREAD_UNLOCK(&mca_oob_tcp_component.tcp_lock);
-            return ORTE_SUCCESS;
-        }
         OPAL_THREAD_UNLOCK(&mca_oob_tcp_component.tcp_lock);
+        return ORTE_SUCCESS;        
+#else
+
+        /* post to a global list of completed messages */
+        if ((msg->msg_flags & ORTE_RML_FLAG_RECURSIVE_CALLBACK) == 0) {
+            int size;
+            OPAL_THREAD_LOCK(&mca_oob_tcp_component.tcp_lock);
+            opal_list_append(&mca_oob_tcp_component.tcp_msg_completed, (opal_list_item_t*)msg);
+            size = opal_list_get_size(&mca_oob_tcp_component.tcp_msg_completed);
+            OPAL_THREAD_UNLOCK(&mca_oob_tcp_component.tcp_lock);
+            if(size > 1) {
+                return ORTE_SUCCESS;
+            }
+        }
 
         /* invoke message callback */
         msg->msg_cbfunc(msg->msg_rc, peer, msg->msg_uiov, msg->msg_ucnt, msg->msg_hdr.msg_tag, msg->msg_cbdata);
 
-        /* dispatch any completed events */
-        OPAL_THREAD_LOCK(&mca_oob_tcp_component.tcp_lock);
-        opal_list_remove_item(&mca_oob_tcp_component.tcp_msg_completed, (opal_list_item_t*)msg);
-        MCA_OOB_TCP_MSG_RETURN(msg);
-        while(NULL != 
-            (item = opal_list_remove_first(&mca_oob_tcp_component.tcp_msg_completed))) {
-            msg = (mca_oob_tcp_msg_t*)item;
-            OPAL_THREAD_UNLOCK(&mca_oob_tcp_component.tcp_lock);
-            msg->msg_cbfunc(
-                msg->msg_rc, 
-                &msg->msg_peer, 
-                msg->msg_uiov, 
-                msg->msg_ucnt, 
-                msg->msg_hdr.msg_tag, 
-                msg->msg_cbdata);
+        /* dispatch any completed events */ 
+        if ((msg->msg_flags & ORTE_RML_FLAG_RECURSIVE_CALLBACK) == 0) {
+            opal_list_item_t* item;
+
             OPAL_THREAD_LOCK(&mca_oob_tcp_component.tcp_lock);
+            opal_list_remove_item(&mca_oob_tcp_component.tcp_msg_completed, (opal_list_item_t*)msg);
+            MCA_OOB_TCP_MSG_RETURN(msg);
+            while(NULL != 
+                  (item = opal_list_remove_first(&mca_oob_tcp_component.tcp_msg_completed))) {
+                msg = (mca_oob_tcp_msg_t*)item;
+                OPAL_THREAD_UNLOCK(&mca_oob_tcp_component.tcp_lock);
+                msg->msg_cbfunc(
+                                msg->msg_rc, 
+                                &msg->msg_peer, 
+                                msg->msg_uiov, 
+                                msg->msg_ucnt, 
+                                msg->msg_hdr.msg_tag, 
+                                msg->msg_cbdata);
+                OPAL_THREAD_LOCK(&mca_oob_tcp_component.tcp_lock);
+                MCA_OOB_TCP_MSG_RETURN(msg);
+            }
+            OPAL_THREAD_UNLOCK(&mca_oob_tcp_component.tcp_lock);
+        } else {
             MCA_OOB_TCP_MSG_RETURN(msg);
         }
-        OPAL_THREAD_UNLOCK(&mca_oob_tcp_component.tcp_lock);
-
+#endif  /* defined(__WINDOWS__) */
     } else {
         opal_condition_broadcast(&msg->msg_condition);
         OPAL_THREAD_UNLOCK(&msg->msg_lock);
@@ -244,11 +273,12 @@ bool mca_oob_tcp_msg_send_handler(mca_oob_tcp_msg_t* msg, struct mca_oob_tcp_pee
             else if (opal_socket_errno == EAGAIN || opal_socket_errno == EWOULDBLOCK)
                 return false;
             else {
-                opal_output(0, "[%lu,%lu,%lu]-[%lu,%lu,%lu] mca_oob_tcp_msg_send_handler: writev failed: %s (%d)", 
-                    ORTE_NAME_ARGS(orte_process_info.my_name), 
-                    ORTE_NAME_ARGS(&(peer->peer_name)), 
+                opal_output(0, "%s->%s mca_oob_tcp_msg_send_handler: writev failed: %s (%d) [sd = %d]", 
+                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), 
+                    ORTE_NAME_PRINT(&(peer->peer_name)), 
                     strerror(opal_socket_errno),
-                    opal_socket_errno);
+                    opal_socket_errno,
+                    peer->peer_sd);
                 mca_oob_tcp_peer_close(peer);
                 msg->msg_rc = ORTE_ERR_CONNECTION_FAILED;
                 return true;
@@ -269,14 +299,14 @@ bool mca_oob_tcp_msg_send_handler(mca_oob_tcp_msg_t* msg, struct mca_oob_tcp_pee
                     return true;
                 }
             }
-        } while(msg->msg_rwnum);
+        } while(1);
     }
 }
 
 /*
  * Receives message data.
- * @param msg the message to be recieved into
- * @param peer the peer to recieve from
+ * @param msg the message to be received into
+ * @param peer the peer to receive from
  * @retval true if the whole message was received
  * @retval false if the whole message was not received
  */
@@ -292,9 +322,9 @@ bool mca_oob_tcp_msg_recv_handler(mca_oob_tcp_msg_t* msg, struct mca_oob_tcp_pee
         if(msg->msg_hdr.msg_size > 0) {
              msg->msg_rwbuf = malloc(msg->msg_hdr.msg_size);
              if(NULL == msg->msg_rwbuf) {
-                 opal_output(0, "[%lu,%lu,%lu]-[%lu,%lu,%lu] mca_oob_tcp_msg_recv_handler: malloc(%d) failed\n", 
-                     ORTE_NAME_ARGS(orte_process_info.my_name),
-                     ORTE_NAME_ARGS(&(peer->peer_name)),
+                 opal_output(0, "%s-%s mca_oob_tcp_msg_recv_handler: malloc(%d) failed\n", 
+                     ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                     ORTE_NAME_PRINT(&(peer->peer_name)),
                      msg->msg_hdr.msg_size);
                  mca_oob_tcp_peer_close(peer);
                  return false;
@@ -307,6 +337,13 @@ bool mca_oob_tcp_msg_recv_handler(mca_oob_tcp_msg_t* msg, struct mca_oob_tcp_pee
              msg->msg_rwiov[1].iov_len = 0;
              msg->msg_rwnum = 0;
         }
+        if (mca_oob_tcp_component.tcp_debug >= OOB_TCP_DEBUG_INFO) {
+            opal_output(0, "%s-%s (origin: %s) mca_oob_tcp_msg_recv_handler: size %lu\n",
+                        ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                        ORTE_NAME_PRINT(&(peer->peer_name)),
+                        ORTE_NAME_PRINT(&(msg->msg_hdr.msg_origin)),
+                        (unsigned long)(msg->msg_hdr.msg_size) );
+        }
     }
 
     /* do the right thing based on the message type */
@@ -343,24 +380,26 @@ static bool mca_oob_tcp_msg_recv(mca_oob_tcp_msg_t* msg, mca_oob_tcp_peer_t* pee
                under UNIX/Linux environments */
             else if (opal_socket_errno == EAGAIN || opal_socket_errno == EWOULDBLOCK)
                 return false;
-            else {
-                opal_output(0, "[%lu,%lu,%lu]-[%lu,%lu,%lu] mca_oob_tcp_msg_recv: readv failed: %s (%d)", 
-                    ORTE_NAME_ARGS(orte_process_info.my_name),
-                    ORTE_NAME_ARGS(&(peer->peer_name)),
-                    strerror(opal_socket_errno),
-                    opal_socket_errno);
-                mca_oob_tcp_peer_close(peer);
-                mca_oob_call_exception_handlers(&peer->peer_name, MCA_OOB_PEER_DISCONNECTED);
-                return false;
+	    opal_output(0, "%s-%s mca_oob_tcp_msg_recv: readv failed: %s (%d)", 
+			ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+			ORTE_NAME_PRINT(&(peer->peer_name)),
+			strerror(opal_socket_errno),
+			opal_socket_errno);
+	    mca_oob_tcp_peer_close(peer);
+            if (NULL != mca_oob_tcp.oob_exception_callback) {
+                mca_oob_tcp.oob_exception_callback(&peer->peer_name, ORTE_RML_PEER_DISCONNECTED);
             }
+	    return false;
         } else if (rc == 0)  {
             if(mca_oob_tcp_component.tcp_debug >= OOB_TCP_DEBUG_CONNECT_FAIL) {
-                opal_output(0, "[%lu,%lu,%lu]-[%lu,%lu,%lu] mca_oob_tcp_msg_recv: peer closed connection", 
-                   ORTE_NAME_ARGS(orte_process_info.my_name),
-                   ORTE_NAME_ARGS(&(peer->peer_name)));
+                opal_output(0, "%s-%s mca_oob_tcp_msg_recv: peer closed connection", 
+                   ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                   ORTE_NAME_PRINT(&(peer->peer_name)));
             }
             mca_oob_tcp_peer_close(peer);
-            mca_oob_call_exception_handlers(&peer->peer_name, MCA_OOB_PEER_DISCONNECTED);
+            if (NULL != mca_oob_tcp.oob_exception_callback) {
+                mca_oob_tcp.oob_exception_callback(&peer->peer_name, ORTE_RML_PEER_DISCONNECTED);
+            }
             return false;
         }
 
@@ -374,10 +413,11 @@ static bool mca_oob_tcp_msg_recv(mca_oob_tcp_msg_t* msg, mca_oob_tcp_peer_t* pee
                 (msg->msg_rwnum)--;
                 (msg->msg_rwptr)++;
                 if(0 == msg->msg_rwnum) {
+		    assert( 0 == rc );
                     return true;
                 }
             }
-        } while(msg->msg_rwnum);
+        } while(1);
     }
     return true;
 }
@@ -399,8 +439,9 @@ void mca_oob_tcp_msg_recv_complete(mca_oob_tcp_msg_t* msg, mca_oob_tcp_peer_t* p
             mca_oob_tcp_msg_data(msg,peer);
             break;
         default:
-            opal_output(0, "[%lu,%lu,%lu] mca_oob_tcp_msg_recv_complete: invalid message type: %d\n",
-                 ORTE_NAME_ARGS(orte_process_info.my_name), msg->msg_hdr.msg_type);
+            opal_output(0, "%s mca_oob_tcp_msg_recv_complete: invalid message type: %d from peer %s\n",
+                        ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), msg->msg_hdr.msg_type, 
+                        ORTE_NAME_PRINT(&peer->peer_name));
             MCA_OOB_TCP_MSG_RETURN(msg);
             break;
     }
@@ -417,10 +458,12 @@ static void mca_oob_tcp_msg_ident(mca_oob_tcp_msg_t* msg, mca_oob_tcp_peer_t* pe
     orte_process_name_t src = msg->msg_hdr.msg_src;
     
     OPAL_THREAD_LOCK(&mca_oob_tcp_component.tcp_lock);
-    if (orte_ns.compare_fields(ORTE_NS_CMP_ALL, &peer->peer_name, &src) != ORTE_EQUAL) {
-        orte_hash_table_remove_proc(&mca_oob_tcp_component.tcp_peers, &peer->peer_name);
+    if (orte_util_compare_name_fields(ORTE_NS_CMP_ALL, &peer->peer_name, &src) != OPAL_EQUAL) {
+        opal_hash_table_remove_value_uint64(&mca_oob_tcp_component.tcp_peers, 
+                                            orte_util_hash_name(&peer->peer_name));
         peer->peer_name = src;
-        orte_hash_table_set_proc(&mca_oob_tcp_component.tcp_peers, &peer->peer_name, peer);
+        opal_hash_table_set_value_uint64(&mca_oob_tcp_component.tcp_peers, 
+                                         orte_util_hash_name(&peer->peer_name), peer);
     }
     OPAL_THREAD_UNLOCK(&mca_oob_tcp_component.tcp_lock);
 }
@@ -446,32 +489,44 @@ static void mca_oob_tcp_msg_data(mca_oob_tcp_msg_t* msg, mca_oob_tcp_peer_t* pee
 {
     /* attempt to match unexpected message to a posted recv */
     mca_oob_tcp_msg_t* post;
+    int rc;
     OPAL_THREAD_LOCK(&mca_oob_tcp_component.tcp_match_lock);
 
+    if (ORTE_JOB_FAMILY(msg->msg_hdr.msg_origin.jobid) !=
+        ORTE_JOB_FAMILY(ORTE_PROC_MY_NAME->jobid)) {
+        /* this message came from a different job family, so we may
+         * not know how to route any reply back to the originator. Update
+         * our route so we can dynamically build the routing table
+         */
+        /* if the origin and the src are the same, then we don't need to do
+         * this - update_route was already called when the connection was
+         * established in oob_tcp_peer
+         */
+        if (OPAL_EQUAL != orte_util_compare_name_fields(ORTE_NS_CMP_ALL, 
+                                                        &(msg->msg_hdr.msg_origin),
+                                                        &(msg->msg_hdr.msg_src))) {
+            if (ORTE_SUCCESS != (rc = orte_routed.update_route(&(msg->msg_hdr.msg_origin),
+                                                               &(msg->msg_hdr.msg_src)))) {
+                /* Nothing we can do about errors here as we definitely want
+                 * the receive to complete, but at least bark loudly
+                 */
+                ORTE_ERROR_LOG(rc);
+            }
+        }
+    }
+    
     /* match msg against posted receives */
-    post = mca_oob_tcp_msg_match_post(&peer->peer_name, msg->msg_hdr.msg_tag);
+    post = mca_oob_tcp_msg_match_post(&msg->msg_hdr.msg_origin, msg->msg_hdr.msg_tag);
     if(NULL != post) {
 
-        if(post->msg_flags & MCA_OOB_ALLOC) {
-
-            /* set the users iovec struct to point to pre-allocated buffer */
-            if(NULL == post->msg_uiov || 0 == post->msg_ucnt) {
-                post->msg_rc = ORTE_ERR_BAD_PARAM;
-            } else {
-                /* first iovec of recv message contains the header -
-                 * subsequent contain user data
-                */
-                post->msg_uiov[0].iov_base = (ompi_iov_base_ptr_t)msg->msg_rwbuf;
-                post->msg_uiov[0].iov_len = msg->msg_hdr.msg_size;
-		        post->msg_rc = msg->msg_hdr.msg_size;
-                msg->msg_rwbuf = NULL;
-            }
-
+        if(NULL == post->msg_uiov || 0 == post->msg_ucnt) {
+            opal_output(0, "msg_data returning bad param");
+            post->msg_rc = ORTE_ERR_BAD_PARAM;
         } else {
-
             /* copy msg data into posted recv */
+            if (post->msg_flags & ORTE_RML_ALLOC) msg->msg_flags |= ORTE_RML_ALLOC;
             post->msg_rc = mca_oob_tcp_msg_copy(msg, post->msg_uiov, post->msg_ucnt);
-            if(post->msg_flags & MCA_OOB_TRUNC) {
+            if(post->msg_flags & ORTE_RML_TRUNC) {
                  int i, size = 0;
                  for(i=1; i<msg->msg_rwcnt+1; i++)
                      size += msg->msg_rwiov[i].iov_len;
@@ -479,7 +534,7 @@ static void mca_oob_tcp_msg_data(mca_oob_tcp_msg_t* msg, mca_oob_tcp_peer_t* pee
             }
         }
 
-        if(post->msg_flags & MCA_OOB_PEEK) {
+        if(post->msg_flags & ORTE_RML_PEEK) {
             /* will need message for actual receive */
             opal_list_append(&mca_oob_tcp_component.tcp_msg_recv, &msg->super.super);
         } else {
@@ -488,7 +543,7 @@ static void mca_oob_tcp_msg_data(mca_oob_tcp_msg_t* msg, mca_oob_tcp_peer_t* pee
         mca_oob_tcp_component.tcp_match_count++;
         OPAL_THREAD_UNLOCK(&mca_oob_tcp_component.tcp_match_lock);
 
-        if(post->msg_flags & MCA_OOB_PERSISTENT) {
+        if(post->msg_flags & ORTE_RML_PERSISTENT) {
             post->msg_cbfunc(
                 post->msg_rc, 
                 &peer->peer_name, 
@@ -497,7 +552,7 @@ static void mca_oob_tcp_msg_data(mca_oob_tcp_msg_t* msg, mca_oob_tcp_peer_t* pee
                 post->msg_hdr.msg_tag, 
                 post->msg_cbdata);
         } else {
-            mca_oob_tcp_msg_complete(post, &peer->peer_name);
+            mca_oob_tcp_msg_complete(post, &msg->msg_hdr.msg_origin);
         }
 
         OPAL_THREAD_LOCK(&mca_oob_tcp_component.tcp_match_lock);
@@ -521,30 +576,38 @@ static void mca_oob_tcp_msg_data(mca_oob_tcp_msg_t* msg, mca_oob_tcp_peer_t* pee
 
 int mca_oob_tcp_msg_copy(mca_oob_tcp_msg_t* msg, struct iovec* iov, int count)
 {
-    int i;
-    unsigned char* src_ptr = (unsigned char*)msg->msg_rwbuf;
+    int i, ret = 0;
+    unsigned char* src_ptr = (unsigned char*) msg->msg_rwbuf;
     size_t src_len = msg->msg_hdr.msg_size;
-    struct iovec *dst = iov;
-    int rc = 0;
 
-    for(i=0; i<count; i++) {
-        unsigned char* dst_ptr = (unsigned char*)dst->iov_base;
-        size_t dst_len = dst->iov_len;
-        while(dst_len > 0) {
-            size_t len = (dst_len <= src_len) ? dst_len : src_len;
-            memcpy(dst_ptr, src_ptr, len);
-            rc += len;
-            dst_ptr += len;
-            dst_len -= len;
-            src_ptr += len;
-            src_len -= len;
-            if(src_len == 0) {
-                return rc;
+    for (i = 0 ; i < count ; i++) {
+        if ((msg->msg_flags & ORTE_RML_ALLOC) && (i == count - 1)) {
+            if (i == 0) {
+                iov[i].iov_base = (IOVBASE_TYPE *) src_ptr;
+                iov[i].iov_len = src_len;
+                msg->msg_rwbuf = NULL;
+            } else {
+                iov[i].iov_base = (IOVBASE_TYPE *) malloc(src_len);
+                iov[i].iov_len = src_len;
+                memcpy(iov[i].iov_base, src_ptr, src_len);
+            }
+        } else {
+            if (iov[i].iov_len > src_len) {
+                memcpy(iov[i].iov_base, src_ptr, src_len);
+                iov[i].iov_len = src_len;
+            } else {
+                memcpy(iov[i].iov_base, src_ptr, iov[i].iov_len);
             }
         }
-        dst++;
+
+        ret += iov[i].iov_len;
+        src_len -= iov[i].iov_len;
+        src_ptr += iov[i].iov_len;
+
+        if (0 == src_len) break;
     }
-    return rc;
+
+    return ret;
 }
 
 /*
@@ -563,7 +626,7 @@ mca_oob_tcp_msg_t* mca_oob_tcp_msg_match_recv(orte_process_name_t* name, int tag
         msg != (mca_oob_tcp_msg_t*) opal_list_get_end(&mca_oob_tcp_component.tcp_msg_recv);
         msg =  (mca_oob_tcp_msg_t*) opal_list_get_next(msg)) {
 
-        if(ORTE_EQUAL == orte_dss.compare(name, &msg->msg_peer, ORTE_NAME)) {
+        if(OPAL_EQUAL == opal_dss.compare(name, &msg->msg_hdr.msg_origin, ORTE_NAME)) {
             if (tag == msg->msg_hdr.msg_tag) {
                 return msg;
             }
@@ -588,9 +651,9 @@ mca_oob_tcp_msg_t* mca_oob_tcp_msg_match_post(orte_process_name_t* name, int tag
         msg != (mca_oob_tcp_msg_t*) opal_list_get_end(&mca_oob_tcp_component.tcp_msg_post);
         msg =  (mca_oob_tcp_msg_t*) opal_list_get_next(msg)) {
 
-        if(ORTE_EQUAL == orte_dss.compare(name, &msg->msg_peer, ORTE_NAME)) {
+        if(OPAL_EQUAL == opal_dss.compare(name, &msg->msg_peer, ORTE_NAME)) {
             if (msg->msg_hdr.msg_tag == tag) {
-                if((msg->msg_flags & MCA_OOB_PERSISTENT) == 0) {
+                if((msg->msg_flags & ORTE_RML_PERSISTENT) == 0) {
                     opal_list_remove_item(&mca_oob_tcp_component.tcp_msg_post, &msg->super.super);
                 }
                 return msg;
@@ -599,7 +662,3 @@ mca_oob_tcp_msg_t* mca_oob_tcp_msg_match_post(orte_process_name_t* name, int tag
     }
     return NULL;
 }
-
-
-
-
